@@ -15,10 +15,6 @@
 #include <qwt_scale_draw.h>
 #include <qwt_plot_renderer.h>
 #include <qdatetime.h>
-#include <qfiledialog.h>
-#include <qimagewriter.h>
-#include <qprintdialog.h>
-#include <qfileinfo.h>
 
 class Grid: public QwtPlotGrid
 {
@@ -26,23 +22,22 @@ public:
     Grid()
     {
         enableXMin( true );
-        setMajPen( QPen( Qt::white, 0, Qt::DotLine ) );
-        setMinPen( QPen( Qt::gray, 0 , Qt::DotLine ) );
+        setMajorPen( Qt::white, 0, Qt::DotLine );
+        setMinorPen( Qt::gray, 0, Qt::DotLine );
     }
 
-    virtual void updateScaleDiv( const QwtScaleDiv &xMap,
-        const QwtScaleDiv &yMap )
+    virtual void updateScaleDiv( const QwtScaleDiv &xScaleDiv,
+        const QwtScaleDiv &yScaleDiv )
     {
-        QList<double> ticks[QwtScaleDiv::NTickTypes];
+        QwtScaleDiv scaleDiv( xScaleDiv.lowerBound(), 
+            xScaleDiv.upperBound() );
 
-        ticks[QwtScaleDiv::MajorTick] =
-            xMap.ticks( QwtScaleDiv::MediumTick );
-        ticks[QwtScaleDiv::MinorTick] =
-            xMap.ticks( QwtScaleDiv::MinorTick );
+        scaleDiv.setTicks( QwtScaleDiv::MinorTick,
+            xScaleDiv.ticks( QwtScaleDiv::MinorTick ) );
+        scaleDiv.setTicks( QwtScaleDiv::MajorTick,
+            xScaleDiv.ticks( QwtScaleDiv::MediumTick ) );
 
-        QwtPlotGrid::updateScaleDiv(
-            QwtScaleDiv( xMap.lowerBound(), xMap.upperBound(), ticks ),
-            yMap );
+        QwtPlotGrid::updateScaleDiv( scaleDiv, yScaleDiv );
     }
 };
 
@@ -80,6 +75,12 @@ Plot::Plot( QWidget *parent ):
     setAxisTitle( QwtPlot::yLeft,
         QString( "Temperature [%1C]" ).arg( QChar( 0x00B0 ) ) );
 
+    QwtPlotCanvas *canvas = new QwtPlotCanvas();
+    canvas->setPalette( Qt::darkGray );
+    canvas->setBorderRadius( 10 );
+
+    setCanvas( canvas );
+
     // grid
     QwtPlotGrid *grid = new Grid;
     grid->attach( this );
@@ -106,7 +107,7 @@ Plot::Plot( QWidget *parent ):
     // RightButton: zoom out by 1
     // Ctrl+RighButton: zoom out to full size
 
-    QwtPlotZoomer* zoomer = new QwtPlotZoomer( canvas() );
+    QwtPlotZoomer* zoomer = new QwtPlotZoomer( canvas );
     zoomer->setRubberBandPen( QColor( Qt::black ) );
     zoomer->setTrackerPen( QColor( Qt::black ) );
     zoomer->setMousePattern( QwtEventPattern::MouseSelect2,
@@ -114,33 +115,29 @@ Plot::Plot( QWidget *parent ):
     zoomer->setMousePattern( QwtEventPattern::MouseSelect3,
         Qt::RightButton );
 
-    QwtPlotPanner *panner = new QwtPlotPanner( canvas() );
+    QwtPlotPanner *panner = new QwtPlotPanner( canvas );
     panner->setMouseButton( Qt::MidButton );
-
-    canvas()->setPalette( Qt::darkGray );
-    canvas()->setBorderRadius( 10 );
 }
 
 QwtScaleDiv Plot::yearScaleDiv() const
 {
     const int days[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-    QList<double> ticks[QwtScaleDiv::NTickTypes];
-
-    QList<double> &mediumTicks = ticks[QwtScaleDiv::MediumTick];
+    QList<double> mediumTicks;
     mediumTicks += 0.0;
     for ( uint i = 0; i < sizeof( days ) / sizeof( days[0] ); i++ )
         mediumTicks += mediumTicks.last() + days[i];
 
-    QList<double> &minorTicks = ticks[QwtScaleDiv::MinorTick];
+    QList<double> minorTicks;
     for ( int i = 1; i <= 365; i += 7 )
         minorTicks += i;
 
-    QList<double> &majorTicks = ticks[QwtScaleDiv::MajorTick];
+    QList<double> majorTicks;
     for ( int i = 0; i < 12; i++ )
         majorTicks += i * 30 + 15;
 
-    QwtScaleDiv scaleDiv( mediumTicks.first(), mediumTicks.last() + 1, ticks );
+    QwtScaleDiv scaleDiv( mediumTicks.first(), mediumTicks.last() + 1, 
+        minorTicks, mediumTicks, majorTicks );
     return scaleDiv;
 }
 
@@ -154,7 +151,7 @@ void Plot::insertCurve( const QString& title,
 
     QwtSymbol *symbol = new QwtSymbol( QwtSymbol::XCross );
     symbol->setSize( 4 );
-    symbol->setPen( QPen( color ) );
+    symbol->setPen( color );
     d_curve->setSymbol( symbol );
 
     d_curve->setSamples( samples );
@@ -168,7 +165,7 @@ void Plot::insertErrorBars(
 {
     d_intervalCurve = new QwtPlotIntervalCurve( title );
     d_intervalCurve->setRenderHint( QwtPlotItem::RenderAntialiased );
-    d_intervalCurve->setPen( QPen( Qt::white ) );
+    d_intervalCurve->setPen( Qt::white );
 
     QColor bg( color );
     bg.setAlpha( 150 );
@@ -207,47 +204,6 @@ void Plot::setMode( int style )
 
 void Plot::exportPlot()
 {
-#ifndef QT_NO_PRINTER
-    QString fileName = "friedberg.pdf";
-#else
-    QString fileName = "friedberg.png";
-#endif
-
-#ifndef QT_NO_FILEDIALOG
-    const QList<QByteArray> imageFormats =
-        QImageWriter::supportedImageFormats();
-
-    QStringList filter;
-    filter += "PDF Documents (*.pdf)";
-#ifndef QWT_NO_SVG
-    filter += "SVG Documents (*.svg)";
-#endif
-    filter += "Postscript Documents (*.ps)";
-
-    if ( imageFormats.size() > 0 )
-    {
-        QString imageFilter( "Images (" );
-        for ( int i = 0; i < imageFormats.size(); i++ )
-        {
-            if ( i > 0 )
-                imageFilter += " ";
-            imageFilter += "*.";
-            imageFilter += imageFormats[i];
-        }
-        imageFilter += ")";
-
-        filter += imageFilter;
-    }
-
-    fileName = QFileDialog::getSaveFileName(
-        this, "Export File Name", fileName,
-        filter.join( ";;" ), NULL, QFileDialog::DontConfirmOverwrite );
-#endif
-    if ( !fileName.isEmpty() )
-    {
-        QwtPlotRenderer renderer;
-        renderer.setDiscardFlag( QwtPlotRenderer::DiscardBackground, false );
-
-        renderer.renderDocument( this, fileName, QSizeF( 300, 200 ), 85 );
-    }
+    QwtPlotRenderer renderer;
+    renderer.exportTo( this, "friedberg.pdf" );
 }
