@@ -13,7 +13,7 @@
 class MyZoomer: public QwtPlotZoomer
 {
 public:
-    MyZoomer( QwtPlotCanvas *canvas ):
+    MyZoomer( QWidget *canvas ):
         QwtPlotZoomer( canvas )
     {
         setTrackerMode( AlwaysOn );
@@ -70,6 +70,7 @@ Plot::Plot( QWidget *parent ):
     d_spectrogram->setRenderThreadCount( 0 ); // use system specific thread count
 
     d_spectrogram->setColorMap( new ColorMap() );
+    d_spectrogram->setCachePolicy( QwtPlotRasterItem::PaintCache );
 
     d_spectrogram->setData( new SpectrogramData() );
     d_spectrogram->attach( this );
@@ -128,7 +129,15 @@ void Plot::showContour( bool on )
 void Plot::showSpectrogram( bool on )
 {
     d_spectrogram->setDisplayMode( QwtPlotSpectrogram::ImageMode, on );
-    d_spectrogram->setDefaultContourPen( on ? QPen() : QPen( Qt::NoPen ) );
+    d_spectrogram->setDefaultContourPen( 
+        on ? QPen( Qt::black, 0 ) : QPen( Qt::NoPen ) );
+
+    replot();
+}
+
+void Plot::setAlpha( int alpha )
+{
+    d_spectrogram->setAlpha( alpha );
     replot();
 }
 
@@ -136,20 +145,22 @@ void Plot::showSpectrogram( bool on )
 
 void Plot::printPlot()
 {
-#if 1
-    QPrinter printer;
-#else
     QPrinter printer( QPrinter::HighResolution );
-#endif
     printer.setOrientation( QPrinter::Landscape );
     printer.setOutputFileName( "spectrogram.pdf" );
+
     QPrintDialog dialog( &printer );
     if ( dialog.exec() )
     {
         QwtPlotRenderer renderer;
 
-        renderer.setDiscardFlag( QwtPlotRenderer::DiscardBackground, false );
-        renderer.setLayoutFlag( QwtPlotRenderer::KeepFrames, true );
+        if ( printer.colorMode() == QPrinter::GrayScale )
+        {
+            renderer.setDiscardFlag( QwtPlotRenderer::DiscardBackground );
+            renderer.setDiscardFlag( QwtPlotRenderer::DiscardCanvasBackground );
+            renderer.setDiscardFlag( QwtPlotRenderer::DiscardCanvasFrame );
+            renderer.setLayoutFlag( QwtPlotRenderer::FrameWithScales );
+        }
 
         renderer.renderTo( this, printer );
     }
