@@ -17,32 +17,26 @@
 #include <qdrawutil.h>
 #include <qstyle.h>
 #include <qstyleoption.h>
+#include <qmath.h>
 
-static inline bool qwtIsLogarithmic( const QwtThermo *thermo )
-{
-    const QwtScaleTransformation* transform =
-        thermo->scaleEngine()->transformation();
-
-    const QwtScaleTransformation::Type scaleType = transform->type();
-
-    delete transform;
-
-    return ( scaleType == QwtScaleTransformation::Log10 );
-}
-
-static inline void qwtDrawLine( 
-    QPainter *painter, int pos, 
-    const QColor &color, const QRect pipeRect, 
+static inline void qwtDrawLine( QPainter *painter, int pos, 
+    const QColor &color, const QRect &pipeRect, const QRect &liquidRect,
     Qt::Orientation orientation )
 {
     painter->setPen( color );
     if ( orientation == Qt::Horizontal )
-        painter->drawLine( pos, pipeRect.top(), pos, pipeRect.bottom() );
+    {
+        if ( pos >= liquidRect.left() && pos < liquidRect.right() )
+            painter->drawLine( pos, pipeRect.top(), pos, pipeRect.bottom() );
+    }
     else
-        painter->drawLine( pipeRect.left(), pos, pipeRect.right(), pos );
+    {
+        if ( pos >= liquidRect.top() && pos < liquidRect.bottom() )
+            painter->drawLine( pipeRect.left(), pos, pipeRect.right(), pos );
+    }
 }
 
-QVector<double> qwtTickList( const QwtScaleDiv &scaleDiv, double value )
+QVector<double> qwtTickList( const QwtScaleDiv &scaleDiv )
 {
     QVector<double> values;
 
@@ -52,12 +46,6 @@ QVector<double> qwtTickList( const QwtScaleDiv &scaleDiv, double value )
     if ( upperLimit < lowerLimit )
         qSwap( lowerLimit, upperLimit );
 
-    if ( value < lowerLimit )
-        return values;
-
-    if ( value < upperLimit )
-        upperLimit = value;
-
     values += lowerLimit;
 
     for ( int tickType = QwtScaleDiv::MinorTick;
@@ -83,17 +71,17 @@ class QwtThermo::PrivateData
 public:
     PrivateData():
         orientation( Qt::Vertical ),
-        scalePos( QwtThermo::LeftScale ),
+        scalePosition( QwtThermo::TrailingScale ),
         spacing( 3 ),
         borderWidth( 2 ),
         pipeWidth( 10 ),
-        minValue( 0.0 ),
-        maxValue( 0.0 ),
-        value( 0.0 ),
         alarmLevel( 0.0 ),
         alarmEnabled( false ),
         autoFillPipe( true ),
-        colorMap( NULL )
+        originMode( QwtThermo::OriginMinimum ),
+        origin( 0.0 ),
+        colorMap( NULL ),
+        value( 0.0 )
     {
         rangeFlags = QwtInterval::IncludeBorders;
     }
@@ -103,23 +91,23 @@ public:
         delete colorMap;
     }
 
-    QwtScaleMap map;
-
     Qt::Orientation orientation;
-    ScalePos scalePos;
+    QwtThermo::ScalePosition scalePosition;
+
     int spacing;
     int borderWidth;
     int pipeWidth;
 
-    double minValue;
-    double maxValue;
     QwtInterval::BorderFlags rangeFlags;
-    double value;
     double alarmLevel;
     bool alarmEnabled;
     bool autoFillPipe;
+    QwtThermo::OriginMode originMode;
+    double origin;
 
     QwtColorMap *colorMap;
+
+    double value;
 };
 
 /*!
@@ -127,10 +115,9 @@ public:
   \param parent Parent widget
 */
 QwtThermo::QwtThermo( QWidget *parent ):
-    QWidget( parent )
+    QwtAbstractScale( parent )
 {
     d_data = new PrivateData;
-    setRange( 0.0, 1.0, false );
 
     QSizePolicy policy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
     if ( d_data->orientation == Qt::Vertical )
@@ -139,6 +126,7 @@ QwtThermo::QwtThermo( QWidget *parent ):
     setSizePolicy( policy );
 
     setAttribute( Qt::WA_WState_OwnSizePolicy, false );
+    layoutThermo( true );
 }
 
 //! Destructor
@@ -180,40 +168,6 @@ QwtInterval::BorderFlags QwtThermo::rangeFlags() const
     return d_data->rangeFlags;
 }
 
-/*!
-  Set the maximum value.
-
-  \param maxValue Maximum value
-  \sa maxValue(), setMinValue(), setRange()
-*/
-void QwtThermo::setMaxValue( double maxValue )
-{
-    setRange( d_data->minValue, maxValue, qwtIsLogarithmic( this ) );
-}
-
-//! Return the maximum value.
-double QwtThermo::maxValue() const
-{
-    return d_data->maxValue;
-}
-
-/*!
-  Set the minimum value.
-
-  \param minValue Minimum value
-  \sa minValue(), setMaxValue(), setRange()
-*/
-void QwtThermo::setMinValue( double minValue )
-{
-    setRange( minValue, d_data->maxValue, qwtIsLogarithmic( this ) );
-}
-
-//! Return the minimum value.
-double QwtThermo::minValue() const
-{
-    return d_data->minValue;
-}
-
 /*!
   Set the current value.
 
@@ -243,12 +197,13 @@ double QwtThermo::value() const
   overload QwtScaleDraw::label().
 
   \param scaleDraw ScaleDraw object, that has to be created with
-                   new and will be deleted in ~QwtThermo or the next
+                   new and will be deleted in ~QwtThermo() or the next
                    call of setScaleDraw().
 */
 void QwtThermo::setScaleDraw( QwtScaleDraw *scaleDraw )
 {
     setAbstractScaleDraw( scaleDraw );
+    layoutThermo( true );
 }
 
 /*!
@@ -270,7 +225,7 @@ QwtScaleDraw *QwtThermo::scaleDraw()
 }
 
 /*!
-  Qt paint event.
+  Paint event handler
   \param event Paint event
 */
 void QwtThermo::paintEvent( QPaintEvent *event )
@@ -286,7 +241,7 @@ void QwtThermo::paintEvent( QPaintEvent *event )
 
     if ( !tRect.contains( event->rect() ) )
     {
-        if ( d_data->scalePos != NoScale )
+        if ( d_data->scalePosition != QwtThermo::NoScale )
             scaleDraw()->draw( &painter, palette() );
     }
 
@@ -302,7 +257,7 @@ void QwtThermo::paintEvent( QPaintEvent *event )
 }
 
 /*! 
-  Qt resize event handler
+  Resize event handler
   \param event Resize event
 */
 void QwtThermo::resizeEvent( QResizeEvent *event )
@@ -332,7 +287,7 @@ void QwtThermo::changeEvent( QEvent *event )
 
 /*!
   Recalculate the QwtThermo geometry and layout based on
-  the QwtThermo::contentsRect() and the fonts.
+  pipeRect() and the fonts.
 
   \param update_geometry notify the layout system and call update
          to redraw the scale
@@ -341,7 +296,7 @@ void QwtThermo::layoutThermo( bool update_geometry )
 {
     const QRect tRect = pipeRect();
     const int bw = d_data->borderWidth + d_data->spacing;
-    const bool inverted = ( maxValue() < minValue() );
+    const bool inverted = ( upperBound() < lowerBound() );
 
     int from, to;
 
@@ -365,28 +320,18 @@ void QwtThermo::layoutThermo( bool update_geometry )
                 to++;
         }
 
-        switch ( d_data->scalePos )
+        if ( d_data->scalePosition == QwtThermo::TrailingScale )
         {
-            case TopScale:
-            {
-                scaleDraw()->setAlignment( QwtScaleDraw::TopScale );
-                scaleDraw()->move( from, tRect.top() - bw );
-                scaleDraw()->setLength( to - from );
-                break;
-            }
-
-            case BottomScale:
-            case NoScale: 
-            default:
-            {
-                scaleDraw()->setAlignment( QwtScaleDraw::BottomScale );
-                scaleDraw()->move( from, tRect.bottom() + bw );
-                scaleDraw()->setLength( to - from );
-                break;
-            }
+            scaleDraw()->setAlignment( QwtScaleDraw::TopScale );
+            scaleDraw()->move( from, tRect.top() - bw );
+        }
+        else
+        {
+            scaleDraw()->setAlignment( QwtScaleDraw::BottomScale );
+            scaleDraw()->move( from, tRect.bottom() + bw );
         }
 
-        d_data->map.setPaintInterval( from, to );
+        scaleDraw()->setLength( to - from );
     }
     else // Qt::Vertical
     {
@@ -408,27 +353,18 @@ void QwtThermo::layoutThermo( bool update_geometry )
                 from--;
         }
 
-        switch ( d_data->scalePos )
+        if ( d_data->scalePosition == QwtThermo::LeadingScale )
         {
-            case RightScale:
-            {
-                scaleDraw()->setAlignment( QwtScaleDraw::RightScale );
-                scaleDraw()->move( tRect.right() + bw, from );
-                scaleDraw()->setLength( to - from );
-                break;
-            }
-
-            case LeftScale:
-            case NoScale: 
-            default:
-            {
-                scaleDraw()->setAlignment( QwtScaleDraw::LeftScale );
-                scaleDraw()->move( tRect.left() - bw, from );
-                scaleDraw()->setLength( to - from );
-                break;
-            }
+            scaleDraw()->setAlignment( QwtScaleDraw::RightScale );
+            scaleDraw()->move( tRect.right() + bw, from );
         }
-        d_data->map.setPaintInterval( to, from );
+        else
+        {
+            scaleDraw()->setAlignment( QwtScaleDraw::LeftScale );
+            scaleDraw()->move( tRect.left() - bw, from );
+        }
+
+        scaleDraw()->setLength( to - from );
     }
 
     if ( update_geometry )
@@ -444,169 +380,152 @@ void QwtThermo::layoutThermo( bool update_geometry )
 */
 QRect QwtThermo::pipeRect() const
 {
-    const QRect cr = contentsRect();
-
     int mbd = 0;
-    if ( d_data->scalePos != NoScale )
+    if ( d_data->scalePosition != QwtThermo::NoScale )
     {
         int d1, d2;
         scaleDraw()->getBorderDistHint( font(), d1, d2 );
         mbd = qMax( d1, d2 );
     }
     const int bw = d_data->borderWidth;
+    const int scaleOff = bw + mbd;
 
-    QRect tRect;
+    const QRect cr = contentsRect();
+
+    QRect pipeRect = cr;
     if ( d_data->orientation == Qt::Horizontal )
     {
-        switch ( d_data->scalePos )
-        {
-            case TopScale:
-            {
-                tRect.setRect(
-                    cr.x() + mbd + bw,
-                    cr.y() + cr.height() - d_data->pipeWidth - bw,
-                    cr.width() - 2 * ( bw + mbd ),
-                    d_data->pipeWidth 
-                );
-                break;
-            }
+        pipeRect.adjust( scaleOff, 0, -scaleOff, 0 );
 
-            case BottomScale:
-            case NoScale: 
-            default:   
-            {
-                tRect.setRect(
-                    cr.x() + mbd + bw,
-                    cr.y() + bw,
-                    cr.width() - 2 * ( bw + mbd ),
-                    d_data->pipeWidth 
-                );
-                break;
-            }
-        }
+        if ( d_data->scalePosition == QwtThermo::TrailingScale )
+            pipeRect.setTop( cr.top() + cr.height() - bw - d_data->pipeWidth );
+        else
+            pipeRect.setTop( bw );
+
+        pipeRect.setHeight( d_data->pipeWidth );
     }
     else // Qt::Vertical
     {
-        switch ( d_data->scalePos )
-        {
-            case RightScale:
-            {
-                tRect.setRect(
-                    cr.x() + bw,
-                    cr.y() + mbd + bw,
-                    d_data->pipeWidth,
-                    cr.height() - 2 * ( bw + mbd ) 
-                );
-                break;
-            }
-            case LeftScale:
-            case NoScale: 
-            default:   
-            {
-                tRect.setRect(
-                    cr.x() + cr.width() - bw - d_data->pipeWidth,
-                    cr.y() + mbd + bw,
-                    d_data->pipeWidth,
-                    cr.height() - 2 * ( bw + mbd ) );
-                break;
-            }
-        }
+        pipeRect.adjust( 0, scaleOff, 0, -scaleOff );
+
+        if ( d_data->scalePosition == QwtThermo::LeadingScale )
+            pipeRect.setLeft( bw );
+        else 
+            pipeRect.setLeft( cr.left() + cr.width() - bw - d_data->pipeWidth );
+
+        pipeRect.setWidth( d_data->pipeWidth );
     }
 
-    return tRect;
+    return pipeRect;
 }
 
 /*!
-   \brief Set the thermometer orientation and the scale position.
+  \brief Set the orientation.
+  \param orientation Allowed values are Qt::Horizontal and Qt::Vertical.
 
-   The scale position NoScale disables the scale.
-   \param o orientation. Possible values are Qt::Horizontal and Qt::Vertical.
-         The default value is Qt::Vertical.
-   \param s Position of the scale.
-         The default value is NoScale.
-
-   A valid combination of scale position and orientation is enforced:
-   - a horizontal thermometer can have the scale positions TopScale,
-     BottomScale or NoScale;
-   - a vertical thermometer can have the scale positions LeftScale,
-     RightScale or NoScale;
-   - an invalid scale position will default to NoScale.
-
-   \sa setScalePosition()
+  \sa orientation(), scalePosition()
 */
-void QwtThermo::setOrientation( Qt::Orientation o, ScalePos s )
+void QwtThermo::setOrientation( Qt::Orientation orientation )
 {
-    if ( o == d_data->orientation && s == d_data->scalePos )
+    if ( orientation == d_data->orientation )
         return;
 
-    switch ( o )
+    d_data->orientation = orientation;
+
+    if ( !testAttribute( Qt::WA_WState_OwnSizePolicy ) )
     {
-        case Qt::Horizontal:
-        {
-            if ( ( s == NoScale ) || ( s == BottomScale ) || ( s == TopScale ) )
-                d_data->scalePos = s;
-            else
-                d_data->scalePos = NoScale;
-            break;
-        }
-        case Qt::Vertical:
-        {
-            if ( ( s == NoScale ) || ( s == LeftScale ) || ( s == RightScale ) )
-                d_data->scalePos = s;
-            else
-                d_data->scalePos = NoScale;
-            break;
-        }
+        QSizePolicy sp = sizePolicy();
+        sp.transpose();
+        setSizePolicy( sp );
+
+        setAttribute( Qt::WA_WState_OwnSizePolicy, false );
     }
 
-    if ( o != d_data->orientation )
-    {
-        if ( !testAttribute( Qt::WA_WState_OwnSizePolicy ) )
-        {
-            QSizePolicy sp = sizePolicy();
-            sp.transpose();
-            setSizePolicy( sp );
-
-            setAttribute( Qt::WA_WState_OwnSizePolicy, false );
-        }
-    }
-
-    d_data->orientation = o;
     layoutThermo( true );
 }
 
 /*!
-  \brief Change the scale position (and thermometer orientation).
-
-  \param scalePos Position of the scale.
-
-  A valid combination of scale position and orientation is enforced:
-  - if the new scale position is LeftScale or RightScale, the
-    scale orientation will become Qt::Vertical;
-  - if the new scale position is BottomScale or TopScale, the scale
-    orientation will become Qt::Horizontal;
-  - if the new scale position is NoScale, the scale orientation 
-    will not change.
-
-  \sa setOrientation(), scalePosition()
+  \return Orientation
+  \sa setOrientation()
 */
-void QwtThermo::setScalePosition( ScalePos scalePos )
+Qt::Orientation QwtThermo::orientation() const
 {
-    if ( ( scalePos == BottomScale ) || ( scalePos == TopScale ) )
-        setOrientation( Qt::Horizontal, scalePos );
-    else if ( ( scalePos == LeftScale ) || ( scalePos == RightScale ) )
-        setOrientation( Qt::Vertical, scalePos );
-    else
-        setOrientation( d_data->orientation, NoScale );
+    return d_data->orientation;
 }
 
 /*!
-   Return the scale position.
+  \brief Change how the origin is determined.
+  \sa originMode(), serOrigin(), origin()
+ */
+void QwtThermo::setOriginMode( OriginMode m )
+{
+    if ( m == d_data->originMode )
+        return;
+
+    d_data->originMode = m;
+    update();
+}
+
+/*!
+  \return Mode, how the origin is determined.
+  \sa setOriginMode(), serOrigin(), origin()
+ */
+QwtThermo::OriginMode QwtThermo::originMode() const
+{
+    return d_data->originMode;
+}
+
+/*!
+  \brief Specifies the custom origin.
+
+  If originMode is set to OriginCustom this property controls where the
+  liquid starts.
+
+  \param origin New origin level
+  \sa setOriginMode(), originMode(), origin()
+ */
+void QwtThermo::setOrigin( double origin )
+{
+    if ( origin == d_data->origin )
+        return;
+
+    d_data->origin = origin;
+    update();
+}
+
+/*!
+  \return Origin of the thermo, when OriginCustom is enabled
+  \sa setOrigin(), setOriginMode(), originMode()
+ */
+double QwtThermo::origin() const
+{
+    return d_data->origin;
+}
+
+/*!
+  \brief Change the position of the scale
+  \param scalePosition Position of the scale.
+
+  \sa ScalePosition, scalePosition()
+*/
+void QwtThermo::setScalePosition( ScalePosition scalePosition )
+{
+    if ( d_data->scalePosition == scalePosition )
+        return;
+
+    d_data->scalePosition = scalePosition;
+
+    if ( testAttribute( Qt::WA_WState_Polished ) )
+        layoutThermo( true );
+}
+
+/*!
+   \return Scale position.
    \sa setScalePosition()
 */
-QwtThermo::ScalePos QwtThermo::scalePosition() const
+QwtThermo::ScalePosition QwtThermo::scalePosition() const
 {
-    return d_data->scalePos;
+    return d_data->scalePosition;
 }
 
 //! Notify a scale change.
@@ -625,20 +544,22 @@ void QwtThermo::drawLiquid(
 {
     painter->save();
     painter->setClipRect( pipeRect, Qt::IntersectClip );
+    painter->setPen( Qt::NoPen );
+
+    const QwtScaleMap scaleMap = scaleDraw()->scaleMap();
+
+    QRect liquidRect = fillRect( pipeRect );
 
-    const bool inverted = ( maxValue() < minValue() );
     if ( d_data->colorMap != NULL )
     {
-        QwtInterval interval( d_data->minValue, d_data->maxValue );
-        interval = interval.normalized();
+        const QwtInterval interval = scaleDiv().interval().normalized();
 
         // Because the positions of the ticks are rounded
         // we calculate the colors for the rounded tick values
 
-        QVector<double> values = qwtTickList(
-            scaleDraw()->scaleDiv(), d_data->value );
+        QVector<double> values = qwtTickList( scaleDraw()->scaleDiv() );
 
-        if ( d_data->map.isInverting() )
+        if ( scaleMap.isInverting() )
             qSort( values.begin(), values.end(), qGreater<double>() );
         else
             qSort( values.begin(), values.end(), qLess<double>() );
@@ -646,78 +567,45 @@ void QwtThermo::drawLiquid(
         int from;
         if ( !values.isEmpty() )
         {
-            from = qRound( d_data->map.transform( values[0] ) );
+            from = qRound( scaleMap.transform( values[0] ) );
             qwtDrawLine( painter, from,
                 d_data->colorMap->color( interval, values[0] ),
-                pipeRect, d_data->orientation );
+                pipeRect, liquidRect, d_data->orientation );
         }
 
         for ( int i = 1; i < values.size(); i++ )
         {
-            const int to = qRound( d_data->map.transform( values[i] ) );
+            const int to = qRound( scaleMap.transform( values[i] ) );
 
             for ( int pos = from + 1; pos < to; pos++ )
             {
-                const double v = d_data->map.invTransform( pos );
+                const double v = scaleMap.invTransform( pos );
 
                 qwtDrawLine( painter, pos, 
                     d_data->colorMap->color( interval, v ),
-                    pipeRect, d_data->orientation );
+                    pipeRect, liquidRect, d_data->orientation );
             }
+
             qwtDrawLine( painter, to,
                 d_data->colorMap->color( interval, values[i] ),
-                pipeRect, d_data->orientation );
+                pipeRect, liquidRect, d_data->orientation );
 
             from = to;
         }
     }
     else
     {
-        const int tval = qRound( d_data->map.transform( d_data->value ) );
-
-        QRect fillRect = pipeRect;
-        if ( d_data->orientation == Qt::Horizontal )
+        if ( !liquidRect.isEmpty() && d_data->alarmEnabled )
         {
-            if ( inverted )
-                fillRect.setLeft( tval );
-            else
-                fillRect.setRight( tval );
-        }
-        else // Qt::Vertical
-        {
-            if ( inverted )
-                fillRect.setBottom( tval );
-            else
-                fillRect.setTop( tval );
-        }
-
-        if ( d_data->alarmEnabled &&
-            d_data->value >= d_data->alarmLevel )
-        {
-            QRect alarmRect = fillRect;
-
-            const int taval = qRound( d_data->map.transform( d_data->alarmLevel ) );
-            if ( d_data->orientation == Qt::Horizontal )
+            const QRect r = alarmRect( liquidRect );
+            if ( !r.isEmpty() )
             {
-                if ( inverted )
-                    alarmRect.setRight( taval );
-                else
-                    alarmRect.setLeft( taval );
+                painter->fillRect( r, palette().brush( QPalette::Highlight ) );
+                liquidRect = QRegion( liquidRect ).subtracted( r ).boundingRect();
             }
-            else
-            {
-                if ( inverted )
-                    alarmRect.setTop( taval );
-                else
-                    alarmRect.setBottom( taval );
-            }
-
-            fillRect = QRegion( fillRect ).subtracted( alarmRect ).boundingRect();
-
-            painter->fillRect( alarmRect, palette().brush( QPalette::Highlight ) );
         }
 
-        painter->fillRect( fillRect, palette().brush( QPalette::ButtonText ) );
+        painter->fillRect( liquidRect, palette().brush( QPalette::ButtonText ) );
     }
 
     painter->restore();
@@ -773,7 +661,7 @@ void QwtThermo::setBorderWidth( int width )
 }
 
 /*!
-   Return the border width of the thermometer pipe.
+   \return Border width of the thermometer pipe.
    \sa setBorderWidth()
 */
 int QwtThermo::borderWidth() const
@@ -781,50 +669,6 @@ int QwtThermo::borderWidth() const
     return d_data->borderWidth;
 }
 
-/*!
-  \brief Set the range
-
-  \param minValue value corresponding lower or left end 
-                  of the thermometer
-  \param maxValue value corresponding to the upper or 
-                  right end of the thermometer
-  \param logarithmic logarithmic mapping, true or false
-*/
-void QwtThermo::setRange( 
-    double minValue, double maxValue, bool logarithmic )
-{
-    if ( minValue == d_data->minValue && maxValue == d_data->maxValue
-        && logarithmic == qwtIsLogarithmic( this ) )
-    {
-        return;
-    }
-
-    if ( logarithmic != qwtIsLogarithmic( this ) )
-    {
-        if ( logarithmic )
-            setScaleEngine( new QwtLog10ScaleEngine );
-        else
-            setScaleEngine( new QwtLinearScaleEngine );
-    }
-
-    d_data->minValue = minValue;
-    d_data->maxValue = maxValue;
-
-    /*
-      There are two different maps, one for the scale, the other
-      for the values. This is confusing and will be changed
-      in the future. TODO ...
-     */
-
-    d_data->map.setTransformation( scaleEngine()->transformation() );
-    d_data->map.setScaleInterval( minValue, maxValue );
-
-    if ( autoScale() )
-        rescale( minValue, maxValue );
-
-    layoutThermo( true );
-}
-
 /*!
   \brief Assign a color map for the fill color
 
@@ -877,10 +721,10 @@ void QwtThermo::setFillBrush( const QBrush& brush )
 }
 
 /*!
-  Return the liquid ( QPalette::ButtonText ) brush. 
+  \return Liquid ( QPalette::ButtonText ) brush. 
   \sa setFillBrush(), QWidget::palette()
 */
-const QBrush& QwtThermo::fillBrush() const
+QBrush QwtThermo::fillBrush() const
 {
     return palette().brush( QPalette::ButtonText );
 }
@@ -904,13 +748,13 @@ void QwtThermo::setAlarmBrush( const QBrush& brush )
 }
 
 /*!
-  Return the liquid brush ( QPalette::Highlight ) above the alarm threshold.
+  \return Liquid brush ( QPalette::Highlight ) above the alarm threshold.
   \sa setAlarmBrush(), QWidget::palette()
 
   \warning The alarm threshold has no effect, when
            a color map has been assigned
 */
-const QBrush& QwtThermo::alarmBrush() const
+QBrush QwtThermo::alarmBrush() const
 {
     return palette().brush( QPalette::Highlight );
 }
@@ -932,7 +776,7 @@ void QwtThermo::setAlarmLevel( double level )
 }
 
 /*!
-  Return the alarm threshold.
+  \return Alarm threshold.
   \sa setAlarmLevel()
 
   \warning The alarm threshold has no effect, when
@@ -959,7 +803,7 @@ void QwtThermo::setPipeWidth( int width )
 }
 
 /*!
-  Return the width of the pipe.
+  \return Width of the pipe.
   \sa setPipeWidth()
 */
 int QwtThermo::pipeWidth() const
@@ -969,14 +813,14 @@ int QwtThermo::pipeWidth() const
 
 /*!
   \brief Enable or disable the alarm threshold
-  \param tf true (disabled) or false (enabled)
+  \param on true (disabled) or false (enabled)
 
   \warning The alarm threshold has no effect, when
            a color map has been assigned
 */
-void QwtThermo::setAlarmEnabled( bool tf )
+void QwtThermo::setAlarmEnabled( bool on )
 {
-    d_data->alarmEnabled = tf;
+    d_data->alarmEnabled = on;
     update();
 }
 
@@ -1001,7 +845,7 @@ QSize QwtThermo::sizeHint() const
 }
 
 /*!
-  \brief Return a minimum size hint
+  \return Minimum size hint
   \warning The return value depends on the font and the scale.
   \sa sizeHint()
 */
@@ -1009,7 +853,7 @@ QSize QwtThermo::minimumSizeHint() const
 {
     int w = 0, h = 0;
 
-    if ( d_data->scalePos != NoScale )
+    if ( d_data->scalePosition != NoScale )
     {
         const int sdExtent = qCeil( scaleDraw()->extent( font() ) );
         const int sdLength = scaleDraw()->minLength( font() );
@@ -1038,3 +882,124 @@ QSize QwtThermo::minimumSizeHint() const
 
     return QSize( w, h );
 }
+
+/*!
+  \brief Calculate the filled rectangle of the pipe
+
+  \param pipeRect Rectangle of the pipe
+  \return Rectangle to be filled ( fill and alarm brush )
+
+  \sa pipeRect(), alarmRect()
+ */
+QRect QwtThermo::fillRect( const QRect &pipeRect ) const
+{
+    double origin;        
+    if ( d_data->originMode == OriginMinimum )
+    {
+        origin = qMin( lowerBound(), upperBound() );
+    }
+    else if ( d_data->originMode == OriginMaximum )
+    {
+        origin = qMax( lowerBound(), upperBound() );
+    }
+    else // OriginCustom
+    {
+        origin = d_data->origin;
+    }
+
+    const QwtScaleMap scaleMap = scaleDraw()->scaleMap();
+
+    int from = qRound( scaleMap.transform( d_data->value ) );
+    int to = qRound( scaleMap.transform( origin ) );
+
+    if ( to < from )
+        qSwap( from, to );
+    
+    QRect fillRect = pipeRect;
+    if ( d_data->orientation == Qt::Horizontal )
+    {
+        fillRect.setLeft( from );
+        fillRect.setRight( to );
+    }
+    else // Qt::Vertical
+    {
+        fillRect.setTop( from );
+        fillRect.setBottom( to );
+    }
+
+    return fillRect.normalized();
+}
+
+/*!
+  \brief Calculate the alarm rectangle of the pipe
+
+  \param fillRect Filled rectangle in the pipe
+  \return Rectangle to be filled with the alarm brush
+
+  \sa pipeRect(), fillRect(), alarmLevel(), alarmBrush()
+ */
+QRect QwtThermo::alarmRect( const QRect &fillRect ) const
+{
+    QRect alarmRect( 0, 0, -1, -1); // something invalid
+
+    if ( !d_data->alarmEnabled )
+        return alarmRect;
+
+    const bool inverted = ( upperBound() < lowerBound() );
+    
+    bool increasing;
+    if ( d_data->originMode == OriginCustom )
+    {
+        increasing = d_data->value > d_data->origin;
+    }
+    else
+    {
+        increasing = d_data->originMode == OriginMinimum;
+    }
+
+    const QwtScaleMap map = scaleDraw()->scaleMap();
+    const int alarmPos = qRound( map.transform( d_data->alarmLevel ) );
+    const int valuePos = qRound( map.transform( d_data->value ) );
+    
+    if ( d_data->orientation == Qt::Horizontal )
+    {
+        int v1, v2;
+        if ( inverted )
+        {
+            v1 = fillRect.left();
+
+            v2 = alarmPos - 1;
+            v2 = qMin( v2, increasing ? fillRect.right() : valuePos );
+        }
+        else
+        {
+            v1 = alarmPos + 1;
+            v1 = qMax( v1, increasing ? fillRect.left() : valuePos );
+
+            v2 = fillRect.right();
+
+        }
+        alarmRect.setRect( v1, fillRect.top(), v2 - v1 + 1, fillRect.height() );
+    }
+    else
+    {
+        int v1, v2;
+        if ( inverted )
+        {
+            v1 = alarmPos + 1;
+            v1 = qMax( v1, increasing ? fillRect.top() : valuePos );
+
+            v2 = fillRect.bottom();
+        }
+        else
+        {
+            v1 = fillRect.top();
+
+            v2 = alarmPos - 1;
+            v2 = qMin( v2, increasing ? fillRect.bottom() : valuePos );
+        }
+        alarmRect.setRect( fillRect.left(), v1, fillRect.width(), v2 - v1 + 1 );
+    }
+
+    return alarmRect;
+} 
